import _ from 'lodash';
import React, {PureComponent} from 'react';
import {ScrollView, StyleSheet} from 'react-native';
import {
  Constants,
  Assets,
  Colors,
  Spacings,
  Keyboard,
  View,
  Text,
  TextField,
  Button,
  Switch
} from 'react-native-ui-lib';
import './demoKeyboards';

const KeyboardAccessoryView = Keyboard.KeyboardAccessoryView;
const KeyboardUtils = Keyboard.KeyboardUtils;
const KeyboardRegistry = Keyboard.KeyboardRegistry;
const TrackInteractive = true;

const demoKeyboards = [
  {
    id: 'unicorn.ImagesKeyboard',
    icon: Assets.icons.demo.image
  },
  {
    id: 'unicorn.CustomKeyboard',
    icon: Assets.icons.demo.dashboard
  }
];

export default class KeyboardAccessoryViewScreen extends PureComponent {
  state = {
    customKeyboard: {
      component: undefined,
      initialProps: undefined
    },
    receivedKeyboardData: undefined,
    useSafeArea: true,
    keyboardOpenState: false
  };

  onKeyboardItemSelected = (keyboardId, params) => {
    const receivedKeyboardData = `onItemSelected from "${keyboardId}"\nreceived params: ${JSON.stringify(params)}`;
    this.setState({receivedKeyboardData});
  };

  onKeyboardResigned = () => {
    this.resetKeyboardView();
  };

  isCustomKeyboardOpen = () => {
    const {keyboardOpenState, customKeyboard} = this.state;
    return keyboardOpenState && !_.isEmpty(customKeyboard);
  };

  resetKeyboardView = () => {
    this.setState({customKeyboard: {}});
  };

  dismissKeyboard = () => {
    KeyboardUtils.dismiss();
  };

  toggleUseSafeArea = () => {
    const {useSafeArea} = this.state;
    this.setState({useSafeArea: !useSafeArea});

    if (this.isCustomKeyboardOpen()) {
      this.dismissKeyboard();
      this.showLastKeyboard();
    }
  };

  showLastKeyboard() {
    const {customKeyboard} = this.state;
    this.setState({customKeyboard: {}});

    setTimeout(() => {
      this.setState({
        keyboardOpenState: true,
        customKeyboard
      });
    }, 500);
  }

  showKeyboardView(component, title) {
    this.setState({
      keyboardOpenState: true,
      customKeyboard: {
        component,
        initialProps: {title}
      }
    });
  }

  onHeightChanged = keyboardAccessoryViewHeight => {
    if (Constants.isIOS) {
      this.setState({keyboardAccessoryViewHeight});
    }
  };

  renderKeyboardAccessoryViewContent = () => {
    return (
      <View style={styles.keyboardContainer} paddingV-s4>
        <View bg-white row spread centerV paddingH-s5 paddingV-s3>
          <TextField
            // migrate
            ref={r => {
              this.textInputRef = r;
            }}
            containerStyle={styles.textField}
            preset={null}
            placeholder={'Message'}
            floatingPlaceholder={false}
            enableErrors={false}
            onFocus={this.resetKeyboardView}
          />
          <Button link grey10 iconSource={Assets.icons.demo.close} onPress={KeyboardUtils.dismiss} marginL-s2/>
        </View>
        <View row paddingH-s4 marginT-s2 spread>
          <View row>
            {demoKeyboards.map(keyboard => (
              <Button
                key={keyboard.id}
                grey10
                link
                iconSource={keyboard.icon}
                onPress={() => this.showKeyboardView(keyboard.id)}
                marginR-s2
              />
            ))}
          </View>

          <Button grey10 label="Reset" link onPress={this.resetKeyboardView}/>
        </View>
      </View>
    );
  };

  requestShowKeyboard = () => {
    KeyboardRegistry.requestShowKeyboard('unicorn.ImagesKeyboard');
  };

  onRequestShowKeyboard = componentID => {
    this.setState({
      customKeyboard: {
        component: componentID,
        initialProps: {title: 'Keyboard 1 opened by button'}
      }
    });
  };

  safeAreaSwitchToggle = () => {
    if (!Constants.isIOS) {
      return;
    }
    const {useSafeArea} = this.state;
    return (
      <View column center>
        <View style={styles.separatorLine}/>
        <View centerV row margin-10>
          <Text text80 grey40>
            Safe Area Enabled:
          </Text>
          <Switch value={useSafeArea} onValueChange={this.toggleUseSafeArea} marginL-14/>
        </View>
        <View style={styles.separatorLine}/>
      </View>
    );
  };

  render() {
    const {message/* , isModal */} = this.props;
    const {receivedKeyboardData, customKeyboard, useSafeArea} = this.state;

    return (
      <View flex bg-grey80>
        <Text h2 marginT-page marginL-page>
          KeyboardAccessoryView + KeyboardRegistry
        </Text>
        <ScrollView
          contentContainerStyle={styles.scrollContainer}
          keyboardDismissMode={TrackInteractive ? 'interactive' : 'none'}
          showsVerticalScrollIndicator={false}
        >
          <Text text40 grey10 marginV-20 center>
            {message}
          </Text>
          <Text testID={'demo-message'}>{receivedKeyboardData}</Text>
          <Button label={'Open Images Keyboard'} link onPress={this.requestShowKeyboard} style={styles.button}/>
          {this.safeAreaSwitchToggle()}
        </ScrollView>

        <KeyboardAccessoryView
          renderContent={this.renderKeyboardAccessoryViewContent}
          onHeightChanged={this.onHeightChanged}
          trackInteractive={TrackInteractive}
          kbInputRef={this.textInputRef}
          kbComponent={customKeyboard.component}
          kbInitialProps={customKeyboard.initialProps}
          onItemSelected={this.onKeyboardItemSelected}
          onKeyboardResigned={this.onKeyboardResigned}
          revealKeyboardInteractive
          onRequestShowKeyboard={this.onRequestShowKeyboard}
          useSafeArea={useSafeArea}
          addBottomView={useSafeArea}
          // usesBottomTabs={!isModal}
        />
      </View>
    );
  }
}

const styles = StyleSheet.create({
  scrollContainer: {
    paddingHorizontal: Spacings.s5,
    flex: 1,
    justifyContent: 'center'
  },
  textField: {
    flex: 1,
    backgroundColor: Colors.grey60,
    paddingVertical: Spacings.s2,
    paddingHorizontal: Spacings.s4,
    borderRadius: 8
  },
  button: {
    padding: Spacings.s2
  },
  keyboardContainer: {
    backgroundColor: Colors.white,
    borderWidth: 1,
    borderColor: Colors.grey60
  },
  separatorLine: {
    flex: 1,
    height: 1,
    backgroundColor: Colors.grey80
  }
});
